/**
 * This work is provided under the terms of the CREATIVE COMMONS PUBLIC
 * LICENSE. This work is protected by copyright and/or other applicable
 * law. Any use of the work other than as authorized under this license
 * or copyright law is prohibited.
 *
 * http://creativecommons.org/licenses/by-nc-sa/2.5/
 *
 * Copyright (C) 2016 OX Software GmbH
 * Mail: info@open-xchange.com
 *
 * @author Daniel Rentz <daniel.rentz@open-xchange.com>
 */

define('io.ox/office/spreadsheet/model/cellstylecollection', [
    'io.ox/office/tk/utils',
    'io.ox/office/editframework/utils/color',
    'io.ox/office/editframework/model/stylecollection',
    'io.ox/office/spreadsheet/utils/sheetutils'
], function (Utils, Color, StyleCollection, SheetUtils) {

    'use strict';

    // class CellStyleCollection ==============================================

    /**
     * Contains the cell style sheets of a spreadsheet document.
     *
     * @constructor
     *
     * @extends StyleCollection
     *
     * @param {SpreadsheetModel} docModel
     *  The document model containing this instance.
     */
    var CellStyleCollection = StyleCollection.extend({ constructor: function (docModel) {

        // self reference
        var self = this;

        // special behavior for OOXML files
        var ooxml = docModel.getApp().isOOXML();

        // the identifier of the built-in 'Hyperlink' style
        var hyperlinkStyleId = null;

        // base constructor ---------------------------------------------------

        StyleCollection.call(this, docModel, 'cell', { poolId: SheetUtils.CELL_POOL_ID, families: 'character apply' });

        // private methods ----------------------------------------------------

        /**
         * Creates a new style sheet, if not existing yet. Updates the category
         * and priority of existing and created style sheets.
         */
        function createStyleSheet(styleId, category, priority, attributeSet, applyFlags) {

            // update category and priority in existing style sheets
            if (self.containsStyleSheet(styleId)) {
                self.setStyleOptions(styleId, { category: category, priority: priority, builtIn: true });
                return;
            }

            // add the apply flags for OOXML files only
            if (ooxml && applyFlags) {
                attributeSet = _.clone(attributeSet);
                attributeSet.apply = applyFlags;
            }

            // create a new (dirty) style sheet, use name as identifier
            self.createDirtyStyleSheet(styleId, styleId, attributeSet, {
                category: category,
                priority: priority,
                hidden: category === 'hidden',
                builtIn: true
            });
        }

        // public methods -----------------------------------------------------

        /**
         * Creates all missing built-in cell styles.
         *
         * @internal
         *  Not intended to be called from any other code than the post-process
         *  handler of the application.
         *
         * @returns {CellStyleCollection}
         *  A reference to this instance.
         */
        this.createMissingStyles = function () {

            function getRGB(value) { return { type: 'rgb', value: value }; }
            function getScheme(value) { return docModel.createJSONColor({ type: 'scheme', value: value }, 'fill'); }
            function getSchemeTint20(value) { return docModel.createJSONColor({ type: 'scheme', value: value }, 'fill', { type: 'tint', value: 20002 }); }
            function getSchemeTint40(value) { return docModel.createJSONColor({ type: 'scheme', value: value }, 'fill', { type: 'tint', value: 40001 }); }
            function getSchemeTint60(value) { return docModel.createJSONColor({ type: 'scheme', value: value }, 'fill', { type: 'tint', value: 60003 }); }

            // the merged attribute set of the default style
            var attributes = this.getDefaultStyleAttributeSet();
            // the default font name of the document
            var fontName = attributes.character.fontName;

            // predefined border values
            var LIGHT_GRAY_BORDER  = { style: 'single', width: 26, color: getRGB('B2B2B2') };
            var MED_GRAY_BORDER    = { style: 'single', width: 26, color: getRGB('7F7F7F') };
            var DARK_GRAY_BORDER   = { style: 'single', width: 26, color: getRGB('3F3F3F') };
            var DOUBLE_GRAY_BORDER = { style: 'double', width: 79, color: getRGB('3F3F3F') };

            // predefined combinations for apply flags (default value for apply flags is true)
            var APPLY_FONT             = { fill: false, border: false, align: false, number: false, protect: false };
            var APPLY_FONT_FILL        = { border: false, align: false, number: false, protect: false };
            var APPLY_FONT_BORDER      = { fill: false, align: false, number: false, protect: false };
            var APPLY_FONT_FILL_BORDER = { align: false, number: false, protect: false };
            var APPLY_FILL_BORDER      = { font: false, align: false, number: false, protect: false };

            // standard styles
            createStyleSheet('Standard', 'markup', 1, {});
            createStyleSheet('Positive', 'markup', 2, { character: { fontName: fontName, color: getRGB('006100') },   cell: { fillType: 'solid', fillColor: getRGB('C6EFCE') } }, APPLY_FONT_FILL);
            createStyleSheet('Neutral',  'markup', 3, { character: { fontName: fontName, color: getRGB('9C6500') },   cell: { fillType: 'solid', fillColor: getRGB('FFEB9C') } }, APPLY_FONT_FILL);
            createStyleSheet('Negative', 'markup', 4, { character: { fontName: fontName, color: getRGB('9C0006') },   cell: { fillType: 'solid', fillColor: getRGB('FFC7CE') } }, APPLY_FONT_FILL);
            createStyleSheet('Note',     'markup', 5, { character: { fontName: fontName, color: getScheme('dark1') }, cell: { fillType: 'solid', fillColor: getRGB('FFFFCC'), borderTop: LIGHT_GRAY_BORDER, borderBottom: LIGHT_GRAY_BORDER, borderLeft: LIGHT_GRAY_BORDER, borderRight: LIGHT_GRAY_BORDER } }, APPLY_FILL_BORDER);

            // heading styles
            createStyleSheet('Title',     'headings', 1, { character: { bold: true, fontName: fontName, fontSize: 18, color: getScheme('dark2') } }, APPLY_FONT);
            createStyleSheet('Heading 1', 'headings', 2, { character: { bold: true, fontName: fontName, fontSize: 15, color: getScheme('dark2') }, cell: { borderBottom: { style: 'single', width: 79, color: getScheme('accent1') } } },        APPLY_FONT_BORDER);
            createStyleSheet('Heading 2', 'headings', 3, { character: { bold: true, fontName: fontName, fontSize: 13, color: getScheme('dark2') }, cell: { borderBottom: { style: 'single', width: 79, color: getSchemeTint40('accent1') } } }, APPLY_FONT_BORDER);
            createStyleSheet('Heading 3', 'headings', 4, { character: { bold: true, fontName: fontName, fontSize: 11, color: getScheme('dark2') }, cell: { borderBottom: { style: 'single', width: 53, color: getSchemeTint40('accent1') } } }, APPLY_FONT_BORDER);
            createStyleSheet('Heading 4', 'headings', 5, { character: { bold: true, fontName: fontName, fontSize: 11, color: getScheme('dark2') } }, APPLY_FONT);
            createStyleSheet('Total',     'headings', 9, { character: { bold: true, fontName: fontName, fontSize: 11, color: getScheme('dark1') }, cell: { borderTop: { style: 'single', width: 26, color: getScheme('accent1') }, borderBottom: { style: 'double', width: 79, color: getScheme('accent1') } } }, APPLY_FONT_BORDER);

            // other content styles
            createStyleSheet('Output',           'hidden', 1, { character: { fontName: fontName, color: getRGB('3F3F3F'), bold: true }, cell: { fillType: 'solid', fillColor: getRGB('F2F2F2'), borderTop: DARK_GRAY_BORDER, borderBottom: DARK_GRAY_BORDER, borderLeft: DARK_GRAY_BORDER, borderRight: DARK_GRAY_BORDER } }, APPLY_FONT_FILL_BORDER);
            createStyleSheet('Calculation',      'hidden', 2, { character: { fontName: fontName, color: getRGB('FA7D00'), bold: true }, cell: { fillType: 'solid', fillColor: getRGB('F2F2F2'), borderTop: MED_GRAY_BORDER,  borderBottom: MED_GRAY_BORDER,  borderLeft: MED_GRAY_BORDER,  borderRight: MED_GRAY_BORDER } }, APPLY_FONT_FILL_BORDER);
            createStyleSheet('Input',            'hidden', 3, { character: { fontName: fontName, color: getRGB('3F3F76') },             cell: { fillType: 'solid', fillColor: getRGB('FFCC99'), borderTop: MED_GRAY_BORDER,  borderBottom: MED_GRAY_BORDER,  borderLeft: MED_GRAY_BORDER,  borderRight: MED_GRAY_BORDER } }, APPLY_FONT_FILL_BORDER);
            createStyleSheet('Explanatory Text', 'hidden', 4, { character: { fontName: fontName, color: getRGB('7F7F7F'), italic: true } }, APPLY_FONT);
            createStyleSheet('Linked Cell',      'hidden', 5, { character: { fontName: fontName, color: getRGB('FA7D00') }, cell: { borderBottom: { style: 'double', width: 17, color: getRGB('FA7D00') } } }, APPLY_FONT_BORDER);
            createStyleSheet('Warning Text',     'hidden', 6, { character: { fontName: fontName, color: getRGB('FF0000') } }, APPLY_FONT);
            createStyleSheet('Check Cell',       'hidden', 7, { character: { fontName: fontName, color: getScheme('light1'), bold: true }, cell: { fillType: 'solid', fillColor: getRGB('A5A5A5'), borderTop: DOUBLE_GRAY_BORDER, borderBottom: DOUBLE_GRAY_BORDER, borderLeft: DOUBLE_GRAY_BORDER, borderRight: DOUBLE_GRAY_BORDER } }, APPLY_FONT_FILL_BORDER);

            // accent styles
            Utils.iterateRange(0, 6, function (index) {
                var styleName = 'Accent ' + (index + 1);
                var colorName = 'accent' + (index + 1);
                createStyleSheet('20% - ' + styleName, 'themes', index,      { character: { fontName: fontName, color: getScheme('dark1') },  cell: { fillType: 'solid', fillColor: getSchemeTint20(colorName) } }, APPLY_FONT_FILL);
                createStyleSheet('40% - ' + styleName, 'themes', index + 6,  { character: { fontName: fontName, color: getScheme('dark1') },  cell: { fillType: 'solid', fillColor: getSchemeTint40(colorName) } }, APPLY_FONT_FILL);
                createStyleSheet('60% - ' + styleName, 'themes', index + 12, { character: { fontName: fontName, color: getScheme('light1') }, cell: { fillType: 'solid', fillColor: getSchemeTint60(colorName) } }, APPLY_FONT_FILL);
                createStyleSheet(styleName,            'themes', index + 18, { character: { fontName: fontName, color: getScheme('light1') }, cell: { fillType: 'solid', fillColor: getScheme(colorName) } },       APPLY_FONT_FILL);
            });

            // hyperlink style
            createStyleSheet('Hyperlink', 'hidden', 99, { character: { fontName: fontName, color: docModel.createJSONColor(Color.HYPERLINK, 'text'), underline: true } }, APPLY_FONT);
            hyperlinkStyleId = this.getStyleIdByName('Hyperlink');

            return this;
        };

        /**
         * Returns the identifier of the built-in 'Hyperlink' cell style sheet.
         *
         * @returns {String}
         *  The identifier of the built-in 'Hyperlink' cell style sheet.
         */
        this.getHyperlinkStyleId = function () {
            return hyperlinkStyleId;
        };

        // initialization -----------------------------------------------------

        // destroy class members on destruction
        this.registerDestructor(function () {
            docModel = self = null;
        });

    } }); // class CellStyleCollection

    // exports ================================================================

    return CellStyleCollection;

});
